/**
 * This code implements the algorithm for longest substring of a given
 * precise density.  The problem is described in the paper
 * "Locating regions in a sequence under density constraints",
 * by Benjamin A. Burton and Mathias Hiron.
 *
 * The algorithm is summarised briefly in the final discussion section
 * of the aforementioned paper.  It based on the DistSort algorithm,
 * described in "Searching a bitstream in linear time for the longest
 * substring of any given density", by Benjamin A. Burton, Algorithmica,
 * in press, doi:10.1007/s00453-010-9424-y .  The main difference is that we
 * use a two-phase radix sort, described in the Burton-Hiron paper above.
 *
 * See README.txt for instructions on how to build and use this code.
 *
 * ------------------------------------------------------------------------
 *
 * This code is copyright (c) 2011, Benjamin Burton <bab@maths.uq.edu.au>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#include <algorithm>
#include <cstdio>
#include <cstring>
#include <iostream>

typedef __int32_t postype;

/**
 * This represents a single point in the graphical representation.
 * Here we make use of the quantity "dist", which is described in the
 * Algorithmica paper by Burton.
 */
struct Coord {
    postype upper;       // dist / c, which is in the range [0, 2n)
    /* postype lower; */ // dist % d, which is in the range [0, d) <= [0, n)
                         // == (p*pos) % d, so don't store it
    /* postype pos; */   // == index into seq[], so don't store it either
};

// Input and output variables:
postype n, c, d, p; // p = d - c
postype ans, ansFrom, ansTo;

// A stable collation by Coord::lower that runs in linear time.
// This is a stable sort by pos % d == (lower / p) % d.
// PRE: The input array is ordered by pos.
// The output array is filled with indices (== pos) into the input array.
void sortLower(Coord* first, Coord* last, postype* dest) {
    postype* count = new postype[d + 1];
    memset(count, 0, (d + 1) * sizeof(postype));

    // Count the number of items with each value of the sort key.
    // Specifically, count[i] will count items with sort key (i - 1).
    // For later convenience, count[0] will be 0.
    Coord* p;
    postype posmod = 0;
    for (p = first, posmod = 0; p < last;
            ++p, posmod = (posmod == d-1 ? 0 : posmod+1))
        ++count[posmod + 1];

    // Convert the count[] array into cumulative sums.
    postype i;
    for (i = 1; i <= d; ++i)
        count[i] += count[i - 1];

    // Fill the destination array.
    for (p = first, posmod = 0; p < last;
            ++p, posmod = (posmod == d-1 ? 0 : posmod+1))
        dest[count[posmod]++] = p - first;

    // Done!
    delete[] count;
};

// Performs a stable sort by Coord::upper that runs in linear time.
// The input array must contain indices into seq[].
//
// The output is not stored; instead we combine this routine with the
// subsequent "what is the longest path?" analysis, and find the
// longest path directly.
void sortUpper(Coord* seq, postype* first, postype* last) {
    ans = 0;

    // The entry latest[i] - 1 takes the last value of the *lower* sort
    // key that has been seen with the upper sort key i, and stores the
    // first position in which this combined (lower, upper) pair has been seen.
    //
    // If the upper sort key i has not yet been seen, then latest[i] = 0.
    postype* latest = new postype[2 * n + 1];
    memset(latest, 0, (2 * n + 1) * sizeof(postype));

    postype* p;
    postype u;
    for (p = first; p < last; ++p) {
        u = seq[*p].upper;
        if (latest[u] == 0 || ((latest[u] - *p) % d) != 0)
            latest[u] = *p;
        else if (*p - latest[u] > ans) {
            ansFrom = latest[u];
            ansTo = *p;
            ans = ansTo - ansFrom;
        }
    }

    // Done!
    delete[] latest;
};

int main(int argc, char* argv[]) {
    n = atol(argv[1]);
    c = atol(argv[2]);
    d = atol(argv[3]);
    p = d - c;

    Coord* seq = new Coord[n + 1];

    // Express dist = n*d as an (upper, lower) pair.
    seq[0].upper = n;

    postype posmod = 0; // Track this separately since % is expensive.
    postype i;
    for (i = 1; i <= n; ++i) {
        // Normally we would set:
        //   seq[i].upper = (dist / static_cast<disttype>(d));
        //   seq[i].lower = (dist % static_cast<disttype>(d));
        // However, we can streamline this by observing that +p / -q
        // only "wraps around" modulo d at most once.
        if (getchar() == '1') {
            // dist += p;
            if (posmod + p < d)
                seq[i].upper = seq[i - 1].upper;
            else
                seq[i].upper = seq[i - 1].upper + 1;
        } else {
            // dist -= c;
            if (posmod >= c)
                seq[i].upper = seq[i - 1].upper;
            else
                seq[i].upper = seq[i - 1].upper - 1;
        }

        posmod += p;
        if (posmod >= d)
            posmod -= d;
    }

    postype* lower = new postype[n + 1];
    sortLower(seq, seq + n + 1, lower);
    sortUpper(seq, lower, lower + n + 1);

    if (ans == 0)
        std::cout << "No suitable substring was found." << std::endl;
    else
        std::cout << "The longest substring is " << ansFrom+1 << ".."
            << ansTo << ", with length " << ans << "." << std::endl;

    delete[] lower;
    return 0;
}
