/**
 * This code implements the algorithm for longest substring with density
 * in a given range.  The problem and algorithm are described in detail
 * in the paper "Locating regions in a sequence under density constraints",
 * by Benjamin A. Burton and Mathias Hiron.
 *
 * See README.txt for instructions on how to build and use this code.
 *
 * ------------------------------------------------------------------------
 *
 * This code is copyright (c) 2011, Benjamin Burton <bab@maths.uq.edu.au>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#include <cstdio>
#include <cstring>
#include <iostream>

typedef __int32_t idxtype;

struct Coord;

// Input variables:
idxtype n;
idxtype c1, d1, p1; // p1 = d1 - c1
idxtype c2, d2, p2; // p2 = d2 - c2

// Look-up tables for multiplying by p1 mod d1 or by p2 mod d2:
idxtype* mult1;
idxtype* mult2;

// Arrays for holding initial and intermediate data:
Coord* seq; // The sequence of points q_0, q_1, ..., q_n
idxtype* sorted;

/**
 * This structure stores a single point q_i.
 *
 * The x coordinate can be obtained as upper1 * d1 + lower1().
 * The y coordinate can be obtained as upper2 * d2 + lower2().
 * The index i can be obtained as idx().
 *
 * The coordinate system in this implementation differs slightly from
 * the Burton-Hiron paper: the coordinates are switched, one
 * coordinate is negated, and the origin is moved to (n*d1, n*d2)
 * (these changes all simplify the implementation).  Specifically:
 *
 *   x = n * d1 - c_1 * i + d_1 * (sum of bits 1..i in the sequence)
 *   y = n * d2 - c_2 * i + d_2 * (sum of bits 1..i in the sequence)
 */
struct Coord {
    idxtype upper1; // upper1 = x / d1, which is in the range [0, 2n).
    idxtype upper2; // upper2 = y / d2, which is in the range [0, 2n).

    // lower1 = x % d1, which is in the range [0, d1).
    // This is always (p1 * idx) % d1, so we don't need to store it.
    inline idxtype lower1() const {
        return mult1[idx() % d1];
    }

    // lower2 = y % d2, which is in the range [0, d2).
    // This is always (p2 * idx) % d2, so we don't need to store it.
    inline idxtype lower2() const {
        return mult2[idx() % d2];
    }

    // idx is just the index into seq[], so we don't need to store this either. 
    inline idxtype idx() const {
        return (this - seq);
    }

    // Equality testing for x coordinates.
    inline bool equal1(const Coord& rhs) const {
        // We simplify the formula for lower1() by assuming gcd(p1, d1) = 1.
        return (upper1 == rhs.upper1 && (idx() - rhs.idx()) % d1 == 0);
    }

    // Comparison function for sorting by x coordinate.
    inline int cmp1(const Coord& rhs) const {
        if (upper1 < rhs.upper1)
            return -1;
        if (upper1 > rhs.upper1)
            return 1;
        if (lower1() < rhs.lower1())
            return -1;
        if (lower1() > rhs.lower1())
            return 1;
        return 0;
    }

    // Comparison function for sorting by y coordinate.
    inline int cmp2(const Coord& rhs) const {
        if (upper2 < rhs.upper2)
            return -1;
        if (upper2 > rhs.upper2)
            return 1;
        if (lower2() < rhs.lower2())
            return -1;
        if (lower2() > rhs.lower2())
            return 1;
        return 0;
    }
};

// A stable sort by x coordinates; that is, by (upper1, lower1) pairs.
//
// PRE: The input array (seq) is ordered by Coord::idx.
// The output array (sorted) will be filled with indices into the input array.
void sort1() {
    idxtype* count = new idxtype[2 * n + 1];
    idxtype* tmp = new idxtype[n + 1];

    const Coord* first = seq;
    const Coord* last = seq + n + 1;
    const Coord* p;
    idxtype i;

    // ----- Stable sort by Coord::lower1 -----

    // Since mods are expensive, track lower1() directly.
    idxtype idxmod;

    // Count the number of items with each value of the sort key.
    // Specifically, count[i] will count the number of items with sort key i-1.
    // For later convenience, count[0] will be 0.
    memset(count, 0, (d1 + 1) * sizeof(idxtype));
    for (p = first, idxmod = 0; p != last;
            ++p, idxmod = (idxmod + p1 >= d1 ? idxmod + p1 - d1 : idxmod + p1))
        ++count[idxmod + 1];

    // Convert the count[] array into cumulative sums.
    for (i = 1; i <= d1; ++i)
        count[i] += count[i - 1];

    // Fill the destination array.
    for (p = first, idxmod = 0; p != last;
            ++p, idxmod = (idxmod + p1 >= d1 ? idxmod + p1 - d1 : idxmod + p1))
        tmp[count[idxmod]++] = p - first;

    // ----- Stable sort by Coord::upper1 -----

    // Same process as before.
    memset(count, 0, (2 * n + 1) * sizeof(idxtype));
    for (p = first; p != last; ++p)
        ++count[p->upper1 + 1];

    for (i = 1; i <= 2 * n; ++i)
        count[i] += count[i - 1];

    // Do the final fill from the tmp array, since our sort must be stable.
    for (i = 0; i < n + 1; ++i)
        sorted[count[seq[tmp[i]].upper1]++] = tmp[i];

    // Done!
    delete[] tmp;
    delete[] count;
}

int main(int argc, char* argv[]) {
    n = atol(argv[1]);
    c1 = atol(argv[2]);
    d1 = atol(argv[3]);
    c2 = atol(argv[4]);
    d2 = atol(argv[5]);
    p1 = d1 - c1;
    p2 = d2 - c2;

    // ----- Set up look-up tables for modular arithmetic -----

    idxtype i;

    mult1 = new idxtype[d1];
    mult1[0] = 0;
    for (i = 1; i < d1; ++i) {
        mult1[i] = mult1[i - 1] + p1;
        if (mult1[i] >= d1)
            mult1[i] -= d1;
    }

    mult2 = new idxtype[d2];
    mult2[0] = 0;
    for (i = 1; i < d2; ++i) {
        mult2[i] = mult2[i - 1] + p2;
        if (mult2[i] >= d2)
            mult2[i] -= d2;
    }

    // ----- Build the sequence of points ---

    seq = new Coord[n + 1];

    // Start at (n*d1, n*d2).
    seq[0].upper1 = seq[0].upper2 = n;

    // Track these separately, since mod is expensive.
    idxtype idxmod1 = 0;
    idxtype idxmod2 = 0;

    for (i = 1; i <= n; ++i) {
        if (getchar() == '1') {
            // dist += p;
            if (idxmod1 + p1 < d1)
                seq[i].upper1 = seq[i - 1].upper1;
            else
                seq[i].upper1 = seq[i - 1].upper1 + 1;

            if (idxmod2 + p2 < d2)
                seq[i].upper2 = seq[i - 1].upper2;
            else
                seq[i].upper2 = seq[i - 1].upper2 + 1;
        } else {
            // dist -= c;
            if (idxmod1 >= c1)
                seq[i].upper1 = seq[i - 1].upper1;
            else
                seq[i].upper1 = seq[i - 1].upper1 - 1;

            if (idxmod2 >= c2)
                seq[i].upper2 = seq[i - 1].upper2;
            else
                seq[i].upper2 = seq[i - 1].upper2 - 1;
        }

        idxmod1 += p1;
        if (idxmod1 >= d1)
            idxmod1 -= d1;
        idxmod2 += p2;
        if (idxmod2 >= d2)
            idxmod2 -= d2;
    }

    // ----- Sort the sequence ---

    // Sort by x coordinte.
    sorted = new idxtype[n + 1];
    sort1();

    // ----- Construct the inner frontier ---

    idxtype* inner = new idxtype[n + 1];
    idxtype nInner = 1;
    inner[0] = sorted[0];

    // Walk through the sequence in increasing order of x, and take
    // points that give us a strictly increasing sequence in y.
    int cmp;
    for (i = 1; i <= n; ++i) {
        cmp = seq[sorted[i]].cmp2(seq[inner[nInner - 1]]);
        if (cmp <= 0)
            continue;

        // We have a greater y coordinate than we've seen before.
        if (seq[sorted[i]].equal1(seq[inner[nInner - 1]])) {
            // Replace: we have the same x coordinate.
            inner[nInner - 1] = sorted[i];
        } else {
            // Append: we have a greater x coordinate than the previous item
            // on the inner frontier.
            inner[nInner++] = sorted[i];
        }
    }

    // ----- Traverse the inner frontier ---

    // We traverse the inner frontier in reverse (since the outer
    // frontier is easier to construct in that direction).
    //
    // As we traverse the inner frontier, maintain a stack of potential
    // best dominators from the outer frontier within a sliding window.
    //
    // Recall that domination for this problem involves (non-strictly)
    // increasing x but decreasing y.
    //
    // The stack of potential best dominators is ordered by (strict)
    // decreasing x, decreasing y, and decreasing idx (all simultaneously).

    idxtype* outer = new idxtype[n + 1];
    idxtype outerBegin = 0;
    idxtype outerEnd = 1;
    outer[0] = sorted[n];

    idxtype outerIt = n - 1; // Next index in sorted[] to consider for the
                             // outer frontier
    idxtype lowest = outer[0]; // The lowest y coordinate seen so far.

    idxtype ans = 0; // Longest path of required density found so far
    idxtype ansFrom = 0, ansTo = 0; // Start and end of best substring

    // Step through the inner frontier:
    idxtype next;
    for (i = nInner - 1; i >= 0; --i) {
        // Expand the list of potential dominators to the left.
        while (outerIt >= 0) {
            // Ensure the next point in sorted[] still dominates inner[i]
            // according to x coordinates.
            next = sorted[outerIt];
            if (seq[inner[i]].cmp1(seq[next]) > 0)
                break;

            --outerIt;

            // Does it have a chance of lying on the outer frontier?
            if (seq[lowest].cmp2(seq[next]) <= 0)
                continue;

            // Place it on the outer frontier for now.
            lowest = next;

            // Does it make other elements in our list obsolete?
            while (outerEnd > outerBegin && next >= outer[outerEnd - 1]) {
                // The new element has greater idx, and will remain in
                // range (dominating points from the inner frontier)
                // for at least as long as the last element of our list.
                --outerEnd;
                continue;
            }

            // Add the new point to our list.
            outer[outerEnd++] = next;
        }

        // Shrink the list of potential denominators from the right.
        while (seq[inner[i]].cmp2(seq[outer[outerBegin]]) < 0)
            ++outerBegin;

        // Fact: the list of potential denominators is non-empty.
        // The first will have greatest idx.
        // Recall also that idx is just the index into seq[].
        if (outer[outerBegin] - inner[i] > ans) {
            ansFrom = inner[i];
            ansTo = outer[outerBegin];
            ans = ansTo - ansFrom;
        }
    }

    // Done!
    if (ans == 0)
        std::cout << "No suitable substring was found." << std::endl;
    else
        std::cout << "The longest substring is " << ansFrom+1 << ".."
            << ansTo << ", with length " << ans << "." << std::endl;

    // ----- Clean up ---

    delete[] outer;
    delete[] inner;
    delete[] sorted;
    delete[] seq;
    delete[] mult2;
    delete[] mult1;
    return 0;
}
