/**
 * This code implements the algorithm for shortest substring with density
 * in a given range.  The problem and algorithm are described in detail
 * in the paper "Locating regions in a sequence under density constraints",
 * by Benjamin A. Burton and Mathias Hiron.
 *
 * See README.txt for instructions on how to build and use this code.
 *
 * ------------------------------------------------------------------------
 *
 * This code is copyright (c) 2011, Benjamin Burton <bab@maths.uq.edu.au>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#include <cstdio>
#include <cstring>
#include <iostream>

#include "libveb.h"

typedef __int32_t idxtype;

struct Coord;

// Input variables:
idxtype n;
idxtype c1, d1, p1; // p1 = d1 - c1
idxtype c2, d2, p2; // p2 = d2 - c2

// Look-up tables for multiplying by p1 mod d1 or by p2 mod d2:
idxtype* mult1;
idxtype* mult2;

// Arrays for initial and intermediate structures:
Coord* seq; // The sequence of points q_0, q_1, ..., q_n
idxtype* sorted;
idxtype* rank;

/**
 * This structure stores a single point q_i.
 *
 * The x coordinate can be obtained as upper1 * d1 + lower1().
 * The y coordinate can be obtained as upper2 * d2 + lower2().
 * The index i can be obtained as idx().
 *
 * The coordinate system in this implementation differs slightly from
 * the Burton-Hiron paper: the coordinates are switched, one
 * coordinate is negated, and the origin is moved to (n*d1, n*d2)
 * (these changes all simplify the implementation).  Specifically:
 *
 *   x = n * d1 - c_1 * i + d_1 * (sum of bits 1..i in the sequence)
 *   y = n * d2 - c_2 * i + d_2 * (sum of bits 1..i in the sequence)
 */
struct Coord {
    idxtype upper1; // upper1 = x / d1, which is in the range [0, 2n).
    idxtype upper2; // upper2 = y / d2, which is in the range [0, 2n).

    // lower1 = x % d1, which is in the range [0, d1).
    // This is always (p1 * idx) % d1, so we don't need to store it.
    inline idxtype lower1() const {
        return mult1[idx() % d1];
    }

    // lower2 = y % d2, which is in the range [0, d2).
    // This is always (p2 * idx) % d2, so we don't need to store it.
    inline idxtype lower2() const {
        return mult2[idx() % d2];
    }

    // idx is just the index into seq[], so we don't need to store this either. 
    inline idxtype idx() const {
        return (this - seq);
    }

    // Comparison function for sorting by x coordinate.
    inline int cmp1(const Coord& rhs) const {
        if (upper1 < rhs.upper1)
            return -1;
        if (upper1 > rhs.upper1)
            return 1;
        if (lower1() < rhs.lower1())
            return -1;
        if (lower1() > rhs.lower1())
            return 1;
        return 0;
    }
};

// Sort by y coordinates; that is, by (upper2, lower2) pairs.
// This is an "inverse" stable sort: if y_i == y_j and i < j, then
// i will appear *after* j in the final sorted list.
//
// PRE: The input array (seq) is ordered by Coord::idx.
// The output array (sorted) will be filled with indices into the input array.
void sort2() {
    idxtype* count = new idxtype[2 * n + 1];
    idxtype* tmp = new idxtype[n + 1];

    const Coord* first = seq;
    const Coord* last = seq + n + 1;
    const Coord* p;
    idxtype i;

    // ----- Inverse stable sort by Coord::lower2 -----

    // Since mods are expensive, track lower2() directly.
    idxtype idxmod;

    // Count the number of items with each value of the sort key.
    // Specifically, count[i] will count the number of items with sort key i-1.
    // For later convenience, count[0] will be 0.
    memset(count, 0, (d2 + 1) * sizeof(idxtype));
    for (p = last, idxmod = seq[n].lower2(); p != first;
            idxmod = (idxmod + c2 >= d2 ? idxmod + c2 - d2 : idxmod + c2)) {
        --p;
        ++count[idxmod + 1];
    }

    // Convert the count[] array into cumulative sums.
    for (i = 1; i <= d2; ++i)
        count[i] += count[i - 1];

    // Fill the destination array.
    for (p = last, idxmod = seq[n].lower2(); p != first;
            idxmod = (idxmod + c2 >= d2 ? idxmod + c2 - d2 : idxmod + c2)) {
        --p;
        tmp[count[idxmod]++] = p - first;
    }

    // ----- Stable sort by Coord::upper2 -----

    // Same process as before.
    memset(count, 0, (2 * n + 1) * sizeof(idxtype));
    for (p = first; p != last; ++p)
        ++count[p->upper2 + 1];

    for (i = 1; i <= 2 * n; ++i)
        count[i] += count[i - 1];

    // Do the final fill from the tmp array, since our sort must be stable.
    for (i = 0; i < n + 1; ++i)
        sorted[count[seq[tmp[i]].upper2]++] = tmp[i];

    // Done!
    delete[] tmp;
    delete[] count;
}

int main(int argc, char* argv[]) {
    // Make sure we can use libveb correctly.
    if (sizeof(uint) < sizeof(idxtype)) {
        std::cerr << "ERROR: uint is too small." << std::endl;
        return 1;
    }

    n = atol(argv[1]);
    c1 = atol(argv[2]);
    d1 = atol(argv[3]);
    c2 = atol(argv[4]);
    d2 = atol(argv[5]);
    p1 = d1 - c1;
    p2 = d2 - c2;

    // ----- Set up look-up tables for modular arithmetic -----

    idxtype i;

    mult1 = new idxtype[d1];
    mult1[0] = 0;
    for (i = 1; i < d1; ++i) {
        mult1[i] = mult1[i - 1] + p1;
        if (mult1[i] >= d1)
            mult1[i] -= d1;
    }

    mult2 = new idxtype[d2];
    mult2[0] = 0;
    for (i = 1; i < d2; ++i) {
        mult2[i] = mult2[i - 1] + p2;
        if (mult2[i] >= d2)
            mult2[i] -= d2;
    }

    // ----- Build the sequence of points ---

    seq = new Coord[n + 1];

    // Start at (n*d1, n*d2).
    seq[0].upper1 = seq[0].upper2 = n;

    // Track these separately, since mod is expensive.
    idxtype idxmod1 = 0;
    idxtype idxmod2 = 0;

    for (i = 1; i <= n; ++i) {
        if (getchar() == '1') {
            // dist += p;
            if (idxmod1 + p1 < d1)
                seq[i].upper1 = seq[i - 1].upper1;
            else
                seq[i].upper1 = seq[i - 1].upper1 + 1;

            if (idxmod2 + p2 < d2)
                seq[i].upper2 = seq[i - 1].upper2;
            else
                seq[i].upper2 = seq[i - 1].upper2 + 1;
        } else {
            // dist -= c;
            if (idxmod1 >= c1)
                seq[i].upper1 = seq[i - 1].upper1;
            else
                seq[i].upper1 = seq[i - 1].upper1 - 1;

            if (idxmod2 >= c2)
                seq[i].upper2 = seq[i - 1].upper2;
            else
                seq[i].upper2 = seq[i - 1].upper2 - 1;
        }

        idxmod1 += p1;
        if (idxmod1 >= d1)
            idxmod1 -= d1;
        idxmod2 += p2;
        if (idxmod2 >= d2)
            idxmod2 -= d2;
    }

    // ----- Sort all y coordinates ---

    sorted = new idxtype[n + 1];
    sort2();

    // The sorted[] array is now a lookup table from rank(y) to idx.
    // Create a reverse lookup table, from idx to rank(y).

    rank = new idxtype[n + 1];
    for (i = 0; i <= n; ++i)
        rank[seq[sorted[i]].idx()] = i;

    // ----- Walk through the sequence and track the partial frontier -----

    idxtype ans = -1;
    idxtype ansFrom = 0, ansTo = 0;

    // The frontier is stored as a set of y coordinate ranks.
    struct Veb frontier = vebnew(n + 1, 0);

    vebput(frontier, rank[0]);

    uint result;
    for (i = 1; i <= n; ++i) {
        // Walk through all points on the frontier that are dominated
        // by seq[i], and simultaneously delete them from the frontier.

        result = rank[i];
        while (1) {
            result = vebsucc(frontier, result + 1);
            if (result == frontier.M)
                break;
            if (seq[i].cmp1(seq[sorted[result]]) < 0)
                break;

            if (ans < 0 || i - seq[sorted[result]].idx() < ans) {
                ansTo = i;
                ansFrom = seq[sorted[result]].idx();
                ans = ansTo - ansFrom;
            }
            vebdel(frontier, result);
        }

        // Insert seq[i] into the frontier.
        vebput(frontier, rank[i]);
    }

    // Done!
    if (ans == -1)
        std::cout << "No suitable substring was found." << std::endl;
    else
        std::cout << "The shortest substring is " << ansFrom+1 << ".."
            << ansTo << ", with length " << ans << "." << std::endl;

    // ----- Clean up ---

    free(frontier.D);
    delete[] rank;
    delete[] sorted;
    delete[] seq;
    delete[] mult2;
    delete[] mult1;
    return 0;
}
